package com.ruoyi.jjh.declaration.entity;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.ruoyi.common.annotation.Excel;
import com.ruoyi.jjh.common.entity.BaseInfoEntity;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.time.LocalDate;

/**
 * 申报模板信息对象 bms_template_info
 *
 * @author farben
 * @date 2023-08-25
 */
@Data
public class BmsTemplateInfo extends BaseInfoEntity {
    private static final long serialVersionUID = 1L;

    /**
     * 主键id
     */
    @TableId(value = "id", type = IdType.AUTO)
    private Long id;

    /**
     * 模板名称
     */
    @NotBlank
    @ApiModelProperty(value = "模板名称")
    @Excel(name = "模板名称")
    private String templateName;


    /**
     * 级别 0:本级,1:省级
     */
    @NotNull
    @ApiModelProperty(value = "级别 0:本级,1:省级")
    @Excel(name = "级别 ", readConverterExp = "0=本级,1=省级")
    private Long level;


    /**
     * 项目大类 1.总部经济 2自主品牌先进技术研究院 3省现代服务业重点项目 4省市服务业领军企业
     * 5.省级现代服务业集聚区 6两业融合试点单位 7楼宇经济 8工业设计中心 9工业设计研究院 10服务型制造示范
     */
    @NotNull
    @ApiModelProperty(value = "项目大类 1.总部经济 2自主品牌先进技术研究院 3省现代服务业重点项目 4省市服务业领军企业 5.省级现代服务业集聚区 6两业融合试点单位 7楼宇经济 8工业设计中心 9工业设计研究院 10服务型制造示范")
    @Excel(name = "项目大类 ", readConverterExp = "1=总部经济,2=自主品牌先进技术研究院,3=省现代服务业重点项目,4=省市服务业领军企业," +
            "5=省级现代服务业集聚区,6=两业融合试点单位,7=楼宇经济,8=工业设计中心,9=工业设计研究院,10=服务型制造示范")
    private Integer projectBigType;

    /**
     * 项目中类
     */
    @NotNull
    @ApiModelProperty(value = "项目中类 1=总部企业,2=自主品牌先进技术研究院,3=省现代服务业重点项目,4=省市服务业领军企业," +
            "            5=省级现代服务业集聚区,6=两业融合试点单位,7=楼宇经济,8=工业设计中心,9=工业设计研究院,10=服务型制造企业,11=服务型制造平台,12=服务型制造示范项目")
    @Excel(name = "项目中类 ", readConverterExp = "1=总部企业,2=自主品牌先进技术研究院,3=省现代服务业重点项目,4=省市服务业领军企业," +
            "5=省级现代服务业集聚区,6=两业融合试点单位,7=楼宇经济,8=工业设计中心,9=工业设计研究院,10=服务型制造企业,11=服务型制造平台,12=服务型制造示范项目")
    private Integer projectMiddleType;

    /**
     * 项目小类
     */
    @NotNull
    @ApiModelProperty(value = "项目小类")
    @Excel(name = "项目小类 ", readConverterExp = "1=省级总部,2=市级总部,3=区级总部,4=自主品牌先进技术研究院,5=省现代服务业重点项目,6=省级服务业领军," +
            "7=市级服务业领军，8=省级现代服务业集聚区,9=两业融合试点单位,10=楼宇经济,11=国家级工业设计中心,12=省级工业设计中心,13=市级工业设计中心," +
            "14=工业设计研究院,15=国家级服务型制造企业,16=省级服务型制造企业,17=市级服务型制造企业,18=区级服务型制造企业,19=国家级服务型制造平台,20=省服务型制造平台," +
            "21=市级服务型制造平台,22=国家级示范项目,23=省级示范项目")
    private Integer projectSmallType;


//    /**
//     * 表单说明
//     */
//    @ApiModelProperty(value = "表单说明")
//    @Excel(name = "表单说明")
//    private String notes;

    /**
     * 责任单位 0：经发委:1：规建委:2：行审局:3：市监局
     */
    @NotNull
    @ApiModelProperty(value = "责任单位 0：经发委:1：规建委:2：行审局:3：市监局")
    @Excel(name = "责任单位", readConverterExp = "0=经发委,1=规建委,2=行审局,3=市监局")
    private Integer responsibilityUnit;

    /**
     * 是否删除：0，未删除；1，已删除
     */
    @ApiModelProperty(value = "是否删除：0，未删除；1，已删除")
    @Excel(name = "是否删除", readConverterExp = "0=未删除,1=已删除")
    private Integer isDeleted;

    /**
     * 模板有效期开始日期
     */
    @NotNull
    @Excel(name = "模板有效期开始日期")
    @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "yyyy-MM-dd", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    @ApiModelProperty("模板有效期开始日期")
    private LocalDate useStart;

    /**
     * 模板有效期结束日期
     */
    @NotNull
    @Excel(name = "模板有效期结束日期")
    @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "yyyy-MM-dd", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    @ApiModelProperty("模板有效期结束日期")
    private LocalDate useEnd;

    /**
     * 状态 0=过期,1=有效
     */
    @Excel(name = "状态 0=过期,1=有效", readConverterExp = "0=过期,1=有效")
    @ApiModelProperty("状态 0=过期,1=有效")
    private Integer status;

    //==============================

    /**
     * 模板json
     */
    @NotBlank
    @ApiModelProperty("模板json")
    private String templateJson;

//    /**
//     * 表名
//     */
//    @ApiModelProperty("表名")
//    private String tableName;
}
