package com.ruoyi.jjh.declaration.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.jjh.declaration.entity.BmsApprovalInfo;
import com.ruoyi.jjh.declaration.entity.BmsEnterpriseBasicInfo;
import com.ruoyi.jjh.declaration.entity.BmsEnterpriseDirectory;
import com.ruoyi.jjh.declaration.entity.BmsTemplateRecord;
import com.ruoyi.jjh.declaration.service.IBmsApprovalInfoService;
import com.ruoyi.jjh.declaration.service.IBmsDeclarationRecordsService;
import com.ruoyi.jjh.declaration.service.IBmsEnterpriseBasicInfoService;
import com.ruoyi.jjh.declaration.service.IBmsEnterpriseDirectoryService;
import com.ruoyi.jjh.declaration.service.IBmsProcessInfoService;
import com.ruoyi.jjh.declaration.service.IBmsTemplateRecordService;
import com.ruoyi.jjh.declaration.service.ICommonService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * <p>
 * 公共服务impl
 * <p>
 *
 * @className: CommonServiceImpl
 * @author: emiya.xie
 * @create: 2023-08-31 15:44
 */
@Service
@Lazy
public class CommonServiceImpl implements ICommonService {


    @Autowired
    private IBmsProcessInfoService iBmsProcessInfoService;

    @Autowired
    private IBmsApprovalInfoService iBmsApprovalInfoService;

    @Autowired
    private IBmsEnterpriseBasicInfoService bmsEnterpriseBasicInfoService;

    @Autowired
    private IBmsEnterpriseDirectoryService bmsEnterpriseDirectoryService;

    @Autowired
    private IBmsTemplateRecordService iBmsTemplateRecordService;

    @Autowired
    private IBmsDeclarationRecordsService iBmsDeclarationRecordsService;


    /**
     * 检测企业信息
     *
     * @param creditCode       企业统一信用代码
     * @param templateRecordId 模板
     * @return
     * @author emiya.xie
     * @create 2023/8/31 15:48
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long checkEnterprise(String creditCode, int templateRecordId) {
        Long enterpriseId = 0L;
        BmsEnterpriseBasicInfo basicInfo = bmsEnterpriseBasicInfoService.getOne(Wrappers.lambdaQuery(BmsEnterpriseBasicInfo.class).eq(BmsEnterpriseBasicInfo::getTyshxydm, creditCode));
        if (BeanUtil.isEmpty(basicInfo)) {
            throw new ServiceException("申请企业未入企业库！");
        }
        enterpriseId = basicInfo.getId();
        BmsEnterpriseDirectory enterpriseDirectory = bmsEnterpriseDirectoryService.getOne(Wrappers.lambdaQuery(BmsEnterpriseDirectory.class).eq(BmsEnterpriseDirectory::getCreditCode, creditCode).eq(BmsEnterpriseDirectory::getTemplateRecordId, templateRecordId));
        if (BeanUtil.isEmpty(enterpriseDirectory)) {
            throw new ServiceException("申请企业不在企业名录中！");
        }
        return enterpriseId;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void checkDeclaration(String creditCode, int templateRecordId) {
        // check当前企业是否已经申请
        BmsEnterpriseBasicInfo enterpriseInfo = bmsEnterpriseBasicInfoService.getOne(Wrappers.lambdaQuery(BmsEnterpriseBasicInfo.class).eq(BmsEnterpriseBasicInfo::getTyshxydm, creditCode));
        // check 申请是否过期或者该申请是不限时的
        BmsTemplateRecord templateRecord = iBmsTemplateRecordService.getById(templateRecordId);
        if (templateRecord.getStatus() == 0) {
            throw new ServiceException("该申请还未到填报时间");
        } else if (templateRecord.getStatus() == 1) {
            throw new ServiceException("该申请已经过期");
        }
//        BmsTemplateRecord templateRecord = iBmsTemplateRecordService.lambdaQuery().eq(BmsTemplateRecord::getId, templateRecordId).le(BmsTemplateRecord::getStartTime, LocalDate.now()).ge(BmsTemplateRecord::getEndTime, LocalDate.now()).one();
//        if (BeanUtil.isEmpty(templateRecord)) {
//            throw new ServiceException("该申请已经错过时间");
//        }
        //查看该企业是否已经填报过
//        BmsTemplateRecord templateRecordOne = iBmsTemplateRecordService.getById(templateRecordId);
//        List<BmsDeclarationRecords> list = iBmsDeclarationRecordsService.lambdaQuery().eq(BmsDeclarationRecords::getEnterpriseId, enterpriseInfo.getId()).eq(BmsDeclarationRecords::getTemplateId, templateRecordOne.getTemplateId()).list();
//        list.forEach(x->{
//            if(x.getDetailId() != null){
//                throw new ServiceException("该企业已经申请，请勿重复申请！");
//            }
//        });
    }

    /**
     * 检测企业信息
     *
     * @param enterpriseId 企业id
     * @param templateId   模板id
     * @return {@link Boolean}
     * @author emiya.xie
     * @create 2023/9/4 22:55
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void checkEnterprise(Long enterpriseId, Long templateId) {
        try {
            BmsEnterpriseBasicInfo basicInfo = bmsEnterpriseBasicInfoService.getOne(Wrappers.lambdaQuery(BmsEnterpriseBasicInfo.class).eq(BmsEnterpriseBasicInfo::getId, enterpriseId));
            if (BeanUtil.isEmpty(basicInfo)) {
                throw new ServiceException("申请企业未入企业库！");
            }
            BmsEnterpriseDirectory enterpriseDirectory = bmsEnterpriseDirectoryService.getOne(Wrappers.lambdaQuery(BmsEnterpriseDirectory.class).eq(BmsEnterpriseDirectory::getCreditCode, basicInfo.getTyshxydm()).eq(BmsEnterpriseDirectory::getTemplateRecordId, templateId));
            if (BeanUtil.isEmpty(enterpriseDirectory)) {
                throw new ServiceException("申请企业不在企业名录中！");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void insertAuditRecord(Long declarationRecordId, Integer approvalType) {
        List<BmsApprovalInfo> list = iBmsApprovalInfoService.lambdaQuery().eq(BmsApprovalInfo::getDeclarationRecordsId, declarationRecordId).list();
        // 新增记录
        BmsApprovalInfo bmsApprovalInfo = new BmsApprovalInfo();
        bmsApprovalInfo.setProcessId(1L);
        bmsApprovalInfo.setDeclarationRecordsId(declarationRecordId);
        bmsApprovalInfo.setApprovalStatus(0);
        bmsApprovalInfo.setSort(list.size() + 1);
        bmsApprovalInfo.setApprovalDept(105L);
        bmsApprovalInfo.setIsReview(0);
        iBmsApprovalInfoService.save(bmsApprovalInfo);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateAuditRecord(Long declarationRecordId, Integer approvalType) {

    }


//    @Override
//    public int insertOpenInterfaceApplyAddDto(OpenInterfaceApplyAddDto openInterfaceApplyAddDto) {
//        int num = 0;
//        // 根据模板id查询对应数据
//        if (openInterfaceApplyAddDto.getDeclarationTemplateType() == 1) {
//            // 项目落户奖补
//            num = iBmsProjectSettlementAwardService.insertBmsProjectSettlementAward(openInterfaceApplyAddDto.getBmsProjectSettlementAwardAddDto());
//        } else if (openInterfaceApplyAddDto.getDeclarationTemplateType() == 2) {
//            // 做大做强奖补
//            num = iBmsBigStrongAwardService.insertBmsBigStrongAward(openInterfaceApplyAddDto.getBmsBigStrongAwardAddDto());
//        } else if (openInterfaceApplyAddDto.getDeclarationTemplateType() == 3) {
//            // 载体建设奖补
//            num = iBmsCarrierConstructionAwardService.insertBmsCarrierConstructionAward(openInterfaceApplyAddDto.getBmsCarrierConstructionAwardAddDto());
//        } else if (openInterfaceApplyAddDto.getDeclarationTemplateType() == 4) {
//            // 平台建设奖补
//            num = iBmsPlatformConstructionAwardService.insertBmsPlatformConstructionAward(openInterfaceApplyAddDto.getBmsPlatformConstructionAwardAddDto());
//        } else if (openInterfaceApplyAddDto.getDeclarationTemplateType() == 5) {
//            // 品牌打造奖补
//            num = iBmsBrandingAwardService.insertBmsBrandingAward(openInterfaceApplyAddDto.getBmsBrandingAwardAddDto());
//        } else if (openInterfaceApplyAddDto.getDeclarationTemplateType() == 6) {
//            // 场景开放奖补
//            num = iBmsSceneOpeningAwardService.insertBmsSceneOpeningAward(openInterfaceApplyAddDto.getBmsSceneOpeningAwardAddDto());
//        } else if (openInterfaceApplyAddDto.getDeclarationTemplateType() == 7) {
//            // 信用管理奖补
//            num = iBmsCreditManagementService.insertBmsCreditManagement(openInterfaceApplyAddDto.getBmsCreditManagementAddDto());
//        } else if (openInterfaceApplyAddDto.getDeclarationTemplateType() == 8) {
//            // 5G+工业互联网奖补
//            num = iBmsIndustrialInternetAwardService.insertBmsIndustrialInternetAward(openInterfaceApplyAddDto.getBmsIndustrialInternetAwardAddDto());
//        } else if (openInterfaceApplyAddDto.getDeclarationTemplateType() == 9) {
//            // 制造服务业有效投入奖补
//            num = iBmsManufacturingServicesAwardService.insertBmsManufacturingServicesAward(openInterfaceApplyAddDto.getBmsManufacturingServicesAwardAddDto());
//        } else if (openInterfaceApplyAddDto.getDeclarationTemplateType() == 10) {
//            // 物流发展奖补
//            num = iBmsLogisticsDevelopmentAwardService.insertBmsLogisticsDevelopmentAward(openInterfaceApplyAddDto.getBmsLogisticsDevelopmentAwardAddDto());
//        } else if (openInterfaceApplyAddDto.getDeclarationTemplateType() == 11) {
//            // 两业融合奖补
//            num = iBmsIntegrationIndustriesService.insertBmsIntegrationIndustries(openInterfaceApplyAddDto.getBmsIntegrationIndustriesAddDto());
//        }
//        return num;
//    }


}
