package com.ruoyi.jjh.ent.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.exception.job.TaskException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.jjh.declaration.regular.TimeChange;
import com.ruoyi.jjh.ent.entity.HomeRequestSmart;
import com.ruoyi.jjh.ent.entity.JSmartDeclaration;
import com.ruoyi.jjh.ent.entity.JSmartReminders;
import com.ruoyi.jjh.ent.entity.request.JAddJobSmart;
import com.ruoyi.jjh.ent.entity.request.JRemindChiefAllRequest;
import com.ruoyi.jjh.ent.entity.request.JSmartRemindersDq;
import com.ruoyi.jjh.ent.entity.request.JSmartRemindersNDq;
import com.ruoyi.jjh.ent.mapper.JSmartRemindersMapper;
import com.ruoyi.jjh.ent.service.JSmartDeclarationService;
import com.ruoyi.jjh.ent.service.JSmartRemindersService;
import com.ruoyi.quartz.domain.SysJob;
import com.ruoyi.quartz.service.ISysJobService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.quartz.SchedulerException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.io.Serializable;

/**
 * 智能提醒（j_smart_reminders）表控制层
 *
 * @author du
 * @since 2024/7/1 13:33
 */
@RestController
@RequestMapping("/jjh/jSmartReminders")
@Api(tags = "智能提醒")
public class JSmartRemindersController extends BaseController {

    @Resource
    private JSmartRemindersService jSmartRemindersService;


    @Autowired
    private ISysJobService jobService;

    @Resource
    private JSmartRemindersMapper jSmartRemindersMapper;

    /**
     * 智能提醒关联表
     */
    @Resource
    private JSmartDeclarationService jSmartDeclarationService;


    @Resource
    private TimeChange timeChange;

    /**
     * 定时器更新不定期提醒数据（暂存）
     *
     * @return 所有数据
     */
    @PreAuthorize("@ss.hasAnyRoles('admin')")
    @ApiOperation(value = "定时器更新不定期提醒数据（暂存）")
    @GetMapping("/updateNoRegularReminders")
    public AjaxResult updateNoRegularReminders() {
        timeChange.changeNdq();
        return success();
    }

    /**
     * 企业端首页智能提醒查询所有数据
     *
     * @return 所有数据
     */
    @PreAuthorize("@ss.hasAnyRoles('ent')")
    @ApiOperation(value = "企业端首页智能提醒查询所有数据", response = JSmartDeclaration.class)
    @GetMapping("/enterpriseAll")
    public AjaxResult enterpriseAll(Page<HomeRequestSmart> page) {
        // 获取当前登录用户是否是企业端
        boolean admin = false;
        String str = null;
        try {
            admin = SecurityUtils.isAdmin(SecurityUtils.getUserId());
            str = SecurityUtils.getUsername();
        } catch (Exception ignored) {
        }
        if (admin) {
            str = null;
        }
        return success(jSmartDeclarationService.selectEnterpriseAllList(page,str));
    }


    /**
     * 政务端首页智能提醒查询所有数据
     *
     * @return 所有数据
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @ApiOperation(value = "政务端首页智能提醒查询数据", response = JSmartDeclaration.class)
    @GetMapping("/chiefAll")
    public AjaxResult chiefAll(Page<HomeRequestSmart> page, JRemindChiefAllRequest req) {
        // 获取当前登录用户是否是政务端
        boolean admin = false;
        try {
            admin = SecurityUtils.isAdmin(SecurityUtils.getUserId());
            req.setUserId(SecurityUtils.getUserId());
        } catch (Exception ignored) {
        }
        if (admin) {
            req.setUserId(null);
        }
        return success(jSmartDeclarationService.chiefAll(page,req));
    }

    /**
     * 智能提醒已读
     *
     * @return 所有数据
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov,ent')")
    @ApiOperation(value = "智能提醒已读")
    @GetMapping("/changeIsRead")
    public AjaxResult changeIsRead(@RequestParam Long id) {
        JSmartDeclaration sma = new JSmartDeclaration();
        sma.setId(id);
        sma.setIsRead(2);
        return success(jSmartDeclarationService.updateById(sma));
    }


    /**
     * 分页查询智能提醒规则数据
     *
     * @param page            分页对象
     * @param jSmartReminders 查询实体
     * @return 所有数据
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @ApiOperation(value = "分页查询智能提醒规则数据", response = JSmartReminders.class)
    @GetMapping
    public AjaxResult selectAll(Page<JSmartReminders> page, JSmartReminders jSmartReminders) {
        return success(jSmartRemindersService.page(page, jSmartReminders));
    }


    /**
     * 通过主键查询单条智能提醒规则数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @ApiOperation(value = "通过主键查询单条智能提醒规则数据", response = JSmartReminders.class)
    @GetMapping("{id}")
    public AjaxResult selectOneSmart(@PathVariable Serializable id) {
        return success(jSmartRemindersService.selectOneSmart(id));
    }

    /**
     * 新增定期提醒数据（政务端和企业端）
     *
     * @param jSmartRemindersDq 实体对象
     * @return 新增结果
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @ApiOperation(value = "新增定期提醒数据（政务端和企业端）")
    @PostMapping
    public AjaxResult insertDs(@RequestBody @Valid JSmartRemindersDq jSmartRemindersDq) throws SchedulerException, TaskException {
        if (jSmartRemindersDq.getProjectId() != null) {
            jSmartRemindersDq.setAlertType(4);
        } else {
            jSmartRemindersDq.setAlertType(1);
        }
        return success(jSmartRemindersService.add(jSmartRemindersDq));
    }


    /**
     * 修改定期提醒数据
     *
     * @param dq 实体对象
     * @return 修改结果
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov')")
    @ApiOperation(value = "修改定期提醒数据")
    @PostMapping("/updateDq")
    public AjaxResult updateDq(@RequestBody @Valid JSmartRemindersDq dq) throws SchedulerException, TaskException {
        if (dq.getAlertType() == 1 || dq.getAlertType() == 4) {
            return success(jSmartRemindersService.updateDq(dq));
        } else {
            throw new ServiceException("请选择正确类型!");
        }
    }

    /**
     * 修改不定期提醒数据
     *
     * @param ndq 实体对象
     * @return 修改结果
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov')")
    @ApiOperation(value = "修改不定期提醒数据")
    @PostMapping("/updateNdq")
    public AjaxResult updateNdq(@RequestBody @Valid JSmartRemindersNDq ndq) {
        if (ndq.getAlertType() == 2 || ndq.getAlertType() == 3) {
            return success(jSmartRemindersService.updateNdq(ndq));
        } else {
            throw new ServiceException("请选择正确类型!");
        }
    }


    /**
     * 删除数据
     *
     * @param id 主键结合
     * @return 删除结果
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov')")
    @ApiOperation(value = "删除数据")
    @PostMapping("/{id}")
    public AjaxResult delete(@PathVariable Long id) throws SchedulerException {
        JSmartReminders js = jSmartRemindersService.getById(id);
        if (js.getAlertManner() == 2) {
            throw new ServiceException("不定期提醒不允许删除");
        }
        JAddJobSmart jAddJobSmart = jSmartRemindersMapper.selectJobSmart(id);
        if (jAddJobSmart.getJobId() != null) {
            SysJob sysJob = new SysJob();
            sysJob.setJobId(jAddJobSmart.getJobId());
            jobService.deleteJob(sysJob);
        }
        return success(jSmartRemindersService.removeById(id));
    }
}
